/* UFO Extender - Mod for X-COM
 * See <http://www.ufopaedia.org/> for more details.
 * Copyright (C) 2008 Seb76
 *
 * This file is part of UFO Extender.

 * UFO Extender is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * UFO Extender is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with UFO Extender.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "xcom.h"
#include "patcher.h"
#include "patches.h"
#include "autosell.h"

/*
 * Called when a facility is destroyed. We update the radar stats to fix the phantom radar bug
 */
void PhantomRadarFix()
{
	UpdateRadarStats(*currentBaseIdx);
}


/*
 * Fix for the base disjoint bug
 * We scan the border modules and fix the tiles when needed
 */
void BaseDisjointFix()
{
	((void (*)())(0x41BF90))();

	//Do not fix if it's not a base defense mission...
	//.text:0041AEF6 66 A1 B4 3E 4A 00             mov     ax, tac_misdata_dat.missionDescriptor
	if(*(short *)0x4A3EB4!=2)
		return;


	unsigned char *modules=&(pGeodata[12]);
	int *groundTiles=&((*pMap)[(*map_numTilesPerLevel)]);

#define MODULE(X,Y) modules[(X)+6*(Y)]
#define TILE(X,Y) groundTiles[(X)+60*(Y)]

	//Fix modules at the bottom
	for(int i=0;i<5;i++)
	{
		int j=5;
		{

			if(MODULE(i,j)>=(unsigned char)0x14)
				continue;
			if(MODULE(i,j)==0x12) //bottom left hangar
				continue;
			if(MODULE(i+1,j)>=(unsigned char)0x14)
				continue;

			char *tile1=(char *)&TILE(i*10+9,j*10+3);
			char *tile2=(char *)&TILE(i*10+9,j*10+4);
			char *tile3=(char *)&TILE(i*10+9,j*10+5);

			char *tile4=(char *)&TILE(i*10+10,j*10+3);
			char *tile5=(char *)&TILE(i*10+10,j*10+4);
			char *tile6=(char *)&TILE(i*10+10,j*10+5);

			char *tile7=(char *)&TILE(i*10+9,j*10+6);

			tile1[0]=9;
			tile1[1]=0;
			tile1[2]=16;
			tile1[3]=0;

			tile2[0]=9;
			tile2[1]=0;
			tile2[2]=0;
			tile2[3]=0;

			tile3[0]=9;
			tile3[1]=0;
			tile3[2]=0;
			tile3[3]=0;

			//Remove west wall
			tile4[1]=0;
			tile5[1]=0;
			tile6[1]=0;

			tile7[2]=16;
		}
	}
	//Fix modules on the right edge
	int i=5;
	{
		for(int j=0;j<5;j++)
		{
			if(MODULE(i,j)>=(unsigned char)0x14)
				continue;
			if(MODULE(i,j)==0x11) //top right hangar
				continue;
			if(MODULE(i,j+1)>=(unsigned char)0x14)
				continue;

			char *tile1=(char *)&TILE(i*10+3,j*10+9);
			char *tile2=(char *)&TILE(i*10+4,j*10+9);
			char *tile3=(char *)&TILE(i*10+5,j*10+9);

			char *tile4=(char *)&TILE(i*10+3,j*10+10);
			char *tile5=(char *)&TILE(i*10+4,j*10+10);
			char *tile6=(char *)&TILE(i*10+5,j*10+10);

			char *tile7=(char *)&TILE(i*10+6,j*10+9);

			tile1[0]=9;
			tile1[1]=15;
			tile1[2]=0;
			tile1[3]=0;

			tile2[0]=9;
			tile2[1]=0;
			tile2[2]=0;
			tile2[3]=0;

			tile3[0]=9;
			tile3[1]=0;
			tile3[2]=0;
			tile3[3]=0;

			//Remove north wall
			tile4[2]=0;
			tile5[2]=0;
			tile6[2]=0;

			tile7[1]=15;
		}
	}
}

/*
 * We replace the standard radar coverage calculation by this one which
 * takes radar stacking into account
 */
void FixRadarStacking(__int16 baseIdx)
{
	unsigned char *base=(*pBaseList)+baseIdx*292;
	unsigned char *modules=&base[22];
	unsigned char *completion=&base[58];
	int smallRadars=0;
	int largeRadars=0;
	int hwd=0;

	//Scan base modules
	for(int i=0;i<6;i++)
	{
		for(int j=0;j<6;j++)
		{
			if(*completion==0)
			{
				switch(*modules)
				{
				case 4://small radar
					smallRadars++;
					break;
				case 5://large radar
					largeRadars++;
					break;
				case 15://hyper wave decoder
					hwd++;
					break;
				}
			}
			modules++;
			completion++;
		}
	}

	float shortDetection=pow(0.9f,smallRadars);
	float largeDetection=pow(0.8f,largeRadars);

	*(short *)(&base[0x10])=(short)((1.0-shortDetection*largeDetection)*100.0);
	*(short *)(&base[0x12])=(short)((1.0-largeDetection)*100.0);
	*(short *)(&base[0x14])=hwd?100:0;
}

/*
 * Queued construction
 */
__declspec(naked) void PatchBaseConstruction()
{
	__asm
	{
		test cx,cx
		jnz can_build

//0x470644
		mov     [eax+16h], dl
		mov     dl, 0xff
		mov     [eax+3Ah], dl
		mov     ecx,1
		push 0x433E36
		//push 0x433E2D //cannot build here msg
		ret
can_build:
		push 0x433E12
		ret
	}
}


__declspec(naked) void PatchBaseConstructionHangar()
{
	__asm
	{
		test si,si
		jnz can_build

//0x470644
		mov     [eax+16h], 0x10
		mov     [eax+17h], 0x11
		mov     [eax+1Ch], 0x12
		mov     [eax+1Dh], 0x13
		mov     dl, 0xff
		mov     [eax+3ah], dl
		mov     [eax+3bh], dl
		mov     [eax+40h], dl
		mov     [eax+41h], dl
		mov     ecx,1
		push 0x433E36
		//push 0x433E2D //cannot build here msg
		ret
can_build:
		push 0x433CF8
		ret
	}
}
__declspec(naked) void PatchValidSquares()
{
	__asm
	{
		mov ebp,[esp+0x14] //&base[0x16]
		mov eax,0x049C2C0 //valid square list (word *)

		mov edi,6*6

squares:
		cmp byte ptr [ebp], 0xff
		jz skip
		cmp byte ptr [ebp+0x24], 0xff
		jnz skip
		mov word ptr [eax],1
skip:
		add eax,2
		inc ebp
		dec edi
		jnz squares


		//tail of the original function
		pop     edi
		pop     esi
		pop     ebp
		pop     ebx
		add     esp, 8
		ret
	}
}

void BaseDrawRemainingDays(int number, int color, __int16 ypos, __int16 xpos, int useSmallFonts)
{
	if(number!=0xff)
	{
		if(number<10)
			xpos+=4;
		DrawNumber2(number, color, ypos, xpos, useSmallFonts);
	}
	else
	{
		DrawString("Queued",color,ypos,xpos-5,1);
	}
}

static unsigned int _edi;

void BaseFacilityCompleted(int baseIdx)
{
	unsigned char *base=(*pBaseList)+baseIdx*292;
	unsigned char *modules=&base[22];
	unsigned char *completion=&base[58];

#define MODULE(x,y) modules[(x)+6*(y)]
#define COMPLETION(x,y) completion[(x)+6*(y)]

	for(int i=0;i<6;i++)
	{
		for(int j=0;j<6;j++)
		{
			if(MODULE(i,j)!=0xff && COMPLETION(i,j)==0xff)
			{
				int unqueue=0;

				if(i!=0 && MODULE(i-1,j)!=0xff && COMPLETION(i-1,j)==0)
					unqueue=1;
				else if(i!=5 && MODULE(i+1,j)!=0xff && COMPLETION(i+1,j)==0)
					unqueue=1;
				else if(j!=0 && MODULE(i,j-1)!=0xff && COMPLETION(i,j-1)==0)
					unqueue=1;
				else if(j!=5 && MODULE(i,j+1)!=0xff && COMPLETION(i,j+1)==0)
					unqueue=1;

				if(unqueue==1)
				{
					//if the newly started construction is _after_ the current one
					//add one to its construction time 'cause it'll be decremented
					//right away

					#define DELTA(i,j) ((((unsigned int)&COMPLETION(i,j))>_edi)?1:0)

					unsigned char *constructionData=(unsigned char *)0x470644;
					switch(MODULE(i,j))
					{
						case 0x10:
							COMPLETION(i,j)=25+DELTA(i,j);
							COMPLETION(i+1,j)=25+DELTA(i+1,j);
							COMPLETION(i,j+1)=25+DELTA(i,j+1);
							COMPLETION(i+1,j+1)=25+DELTA(i+1,j+1);
							break;
						case 0x11:
							COMPLETION(i-1,j)=25+DELTA(i-1,j);
							COMPLETION(i,j)=25+DELTA(i,j);
							COMPLETION(i-1,j+1)=25+DELTA(i-1,j+1);
							COMPLETION(i,j+1)=25+DELTA(i,j+1);
							break;
						case 0x12:
							COMPLETION(i,j-1)=25+DELTA(i,j-1);
							COMPLETION(i+1,j-1)=25+DELTA(i+1,j-1);
							COMPLETION(i,j)=25+DELTA(i,j);
							COMPLETION(i+1,j)=25+DELTA(i+1,j);
							break;
						case 0x13:
							COMPLETION(i-1,j-1)=25+DELTA(i-1,j-1);
							COMPLETION(i,j-1)=25+DELTA(i,j-1);
							COMPLETION(i-1,j)=25+DELTA(i-1,j);
							COMPLETION(i,j)=25+DELTA(i,j);
							break;
						default:
							COMPLETION(i,j)=(*(constructionData+16*MODULE(i,j)))+DELTA(i,j);
							break;
					}
				}
			}
		}
	}

	UpdateRadarStats(baseIdx);
}
__declspec(naked) void BaseFacilityCompletedStub(int baseIdx)
{
	__asm
	{
		mov _edi,edi
		jmp BaseFacilityCompleted
	}
}

//Base build stacking
void ModBaseBuildingStacking()
{
	InsertJmp(0x433E0D,PatchBaseConstruction);
	InsertJmp(0x433CF3,PatchBaseConstructionHangar);
	InsertJmp(0x43A41C,PatchValidSquares);
	InsertCall(0x433651,BaseDrawRemainingDays);
	InsertCall(0x441AAC,BaseFacilityCompletedStub);
}

void BaseFacilityDismantleConstructionCrash()
{
	char patch[]={0x83}; 
	PatchInPlace(0x441ABA,patch,1); //text 00441AB9:jge     loc_441C6A-->jle     loc_441C6A
}

/*
00 - Access Lift
01 - Living Quarters
02 - Laboratory
03 - Workshop
04 - Small Radar System
05 - Large Radar System
06 - Missile Defense
07 - General Stores
08 - Alien Containment
09 - Laser Defense
0A - Plasma Defense
0B - Fusion Ball Defense
0C - Grav Shield
0D - Mind Shield
0E - Psionic Laboratory
0F - Hyper-wave Decoder
10 - Hangar (Top Left)
11 - Hangar (Top Right)
12 - Hangar (Bottom Left)
13 - Hangar (Bottom Right)
FF - Empty
*/
static struct {
	const char *name;
	unsigned int moduleId;
} map[]=
{
	{"AccessLift",0},
	{"LivingQuarters",1},
	{"Laboratory",2},
	{"Workshop",3},
	{"SmallRadar",4},
	{"LargeRadar",5},
	{"MissileDefense",6},
	{"GeneralStores",7},
	{"AlienContainment",8},
	{"LaserDefense",9},
	{"PlasmaDefense",10},
	{"FusionBallDefense",11},
	{"GravShield",12},
	{"MindShield",13},
	{"PsionicLaboratory",14},
	{"HyperWaveDecoder",15},
	{"HangarTL",16},
	{"HangarTR",17},
	{"HangarBL",18},
	{"HangarBR",19},
	{"Empty",255},
};
static unsigned char nameToId(const char *name)
{
	for(int i=0;i<21;i++)
	{
		if(!strcmp(name,map[i].name))
		{
			return map[i].moduleId;
		}
	}
	return 0xff;
}

void SafeBaseDesign()
{
	char modules[6][6][1024];
	
	for(int i=0;i<6;i++)
	{
		char row[16];
		char line[1024];
		sprintf(row,"row%d",i+1);
		INI_READ_STRING("Initial Base",row,"",line,1024);
		sscanf(line,"%s %s %s %s %s %s",modules[i][0],modules[i][1],modules[i][2],modules[i][3],modules[i][4],modules[i][5]);
	}
	for(int i=0;i<6;i++)
	{
		for(int j=0;j<6;j++)
		{
			unsigned char id=nameToId(modules[i][j]);
			unsigned char used=id!=0xff?0:0xff;

			PatchInPlace((int)firstBaseTemplate+0x16+i*6+j,&id,1);
			PatchInPlace((int)firstBaseTemplate+0x3A+i*6+j,&used,1);
		}
	}
}

void SwitchSoldiers(unsigned char *a, unsigned char *b)
{
	//size of SOLDIER.DAT entry = 68
	unsigned char tmp[68];
	memcpy(tmp,a,68);
	memcpy(a,b,68);
	memcpy(b,tmp,68);
}

void CrewDlgAddUpDownButton(const char *str, int color, __int16 ypos, __int16 xpos, int smallFont)
{
	DrawString(str, color, ypos, xpos, smallFont);

	AddDialogButton(0xC4+8, 2);
	AddDialogButton(0xD0+8, 3);
}

int GetItemEntryFromId(int id)
{
	for(int i=0;i<*DlgNumItems;i++)
	{
		if(DlgItemIdList[i]==id)
			return i;
	}
	return -1;
}

void CrewHook()
{
	int now = timeGetTime();
	int moveOnTop = 0;

	WaitTilMouseButtonsAreReleased();

	if(timeGetTime()-now>200)
		moveOnTop=1;

	if(*DlgDialog_ClickedItem==0xffff)
		return;

	unsigned char *currentSoldier=*pSoldierDat+68**DlgDialog_ClickedItem;

	if(*mouse_xpos<128)
	{
		unsigned short baseIdx = *(unsigned short *)(currentSoldier+2);
		ShowSoldierStat(*DlgDialog_ClickedItem, baseIdx);
		*DlgDialog_ClickedItem=0xffff;
		return;
	}

	if(*DlgUpperLowerButtonPressed)
	{
		int entry=GetItemEntryFromId(*DlgDialog_ClickedItem);

		if(*DlgUpperLowerButtonPressed==4) //down
		{
			if(entry<*DlgNumItems-1)
			{
				if(moveOnTop)
				{
					while(entry<*DlgNumItems-1)
					{
						unsigned char *current = *pSoldierDat+68*DlgItemIdList[entry];
						unsigned char *next = *pSoldierDat+68*DlgItemIdList[entry+1];
						SwitchSoldiers(current, next);
						entry++;
					}
				}
				else
				{
					unsigned char *nextSoldier=*pSoldierDat+68*DlgItemIdList[entry+1];
					SwitchSoldiers(currentSoldier,nextSoldier);
				}
			}
		}
		else if(*DlgUpperLowerButtonPressed==3) //up
		{
			if(entry>0)
			{
				if(moveOnTop)
				{
					while(entry>0)
					{
						unsigned char *current = *pSoldierDat+68*DlgItemIdList[entry];
						unsigned char *previous = *pSoldierDat+68*DlgItemIdList[entry-1];
						SwitchSoldiers(current, previous);
						entry--;
					}
				}
				else
				{
					unsigned char *previousSoldier=*pSoldierDat+68*DlgItemIdList[entry-1];
					SwitchSoldiers(currentSoldier,previousSoldier);
				}
			}
		}

		*DlgDialog_ClickedItem=0xffff;
	}
}

__declspec(naked) void DisableSoldierPurchase(void)
{
	__asm
	{
		mov ecx, 0x470762+0x14
		mov eax,1
		ret
	}
}

__declspec(naked) void FixScientistsEngineersOverflow(void)
{
	__asm
	{
		mov     dl, [ecx]
		add     dl, al
		jnc     nooverflow
		mov     dl, 255
nooverflow:
		mov     [ecx], dl
		ret
	}
}

//.text:00448E22 A1 6C 3F 4A 00                mov     eax, carretPosition
//.text:00448E27 8B 4F 0A                      mov     ecx, [edi+0Ah]

__declspec(naked) void IncomeOnlyItemPrice(void)
{
	__asm
	{
		mov eax, 0x4A3F6C
		mov eax, [eax]
		mov ecx, [edi+0xA]
		cmp byte ptr [edi+18],1
		jz purchaseable
		xor ecx,ecx
purchaseable:
		ret
	}
}

//.text:004489AE 8B 90 5A 07 47 00             mov     edx, [eax+47075Ah] //sell price (+8 for purchaseable)

__declspec(naked) void IncomeOnlyItemPrice2(void)
{
	__asm
	{
		mov dl, [eax+0x47075A+8] //purchaseability
		cmp dl, 1
		mov edx, [eax+0x47075A] //sell price
		jz purchaseable
		xor edx,edx
purchaseable:
		ret
	}
}

//.text:0044892B 8B 96 5A 07 47 00             mov     edx, purchaseInfo.sellPrice[esi]
__declspec(naked) void IncomeOnlyItemPrice3(void)
{
	__asm
	{
		mov dl, [esi+0x47075A+8] //purchaseability
		cmp dl, 1
		mov edx, [esi+0x47075A] //sell price
		jz purchaseable
		xor edx,edx
purchaseable:
		ret
	}
}

void CreateCraftDeequipButton(int ypos, int xpos, int height, int width, int color, char *str)
{
//		int __cdecl CreateButton(int ypos, int xpos, int height, int width, int color, char *str)
	CreateButton(ypos, xpos-8, height, width, color, str);
	CreateButton(*(short *)0x49B952+*(short *)0x49A83E,*(short *)0x49B90E+0x28+0x46+8,0x10,0x46,*(char *)0x49AA5E,"DE-EQUIP");
}

__declspec(naked) void ExitCraftArmament(void)
{
	__asm
	{
//		 .data:0049B860 ?? ??                         DlgClickedButton dw ?                   ; DATA XREF: BuildFacilities+5Dr

		push esi
		mov esi, 0x49B860 //Clicked button
		cmp word ptr [esi], 1
		pop esi
		jz short cancel

		push edi
		mov edi,[esp+12]
		mov eax,0x4569E5
		jmp eax
cancel:
		pop esi
		ret
	}
}

/*
.text:00434D47 8A 04 0A                      mov     al, [edx+ecx]
.text:00434D4A 84 C0                         test    al, al
*/
__declspec(naked) int AssignAllScientists()
{
	//ebp = available scientist
	//ebx = available lab space
	//[edx+ecx] = assigned scientists
	__asm
	{
		mov		al,[edx+ecx]
		test	al,al
		jz		assign_all
		ret
assign_all:
		mov		[esp+14],1
		cmp		ebx, ebp
		jl		lab_space_full
//enough space for everyone
		mov		eax, ebp
		mov		[edx+ecx], al
		sub		ebx, ebp
		xor		ebp,ebp
		test	ebp,ebp
		ret
lab_space_full:
		mov		[edx+ecx], bl
		sub		ebp,ebx
		xor		ebx,ebx
		test	ebx,ebx
		ret
	}
}

__declspec(naked) void AssignAllScientists2()  //combined assign all with Human Limits
{
	//ebp = available scientist
	//ebx = available lab space
	//[edx+ecx] = assigned scientists
	__asm
	{
		cmp     ecx, 78h
		jl      skip
		cmp     ecx, 86h
		jg      skip
		mov     ecx, dword ptr ds:49AB30h
		mov     al, byte ptr [edx+ecx]
		test    al, al
		jz      AssignAll
		cmp     ebp, 0//
		jz      skip//
		dec     al
		dec     al
		inc     ebp             
		inc     ebp             // scientist
		mov     byte ptr [edx+ecx], al
		inc     ebx
		mov     dword ptr [esp+14h], esi //var_8]
skip:
		mov     eax, 434D63h
		jmp     eax
AssignAll:
		mov     dword ptr [esp+14h],1
		mov     eax, ebp
		and     eax, 0FFh
		shr     eax, 1
		cmp     bx, ax
		jl      FULL
		sub     ebx, eax
		shl     eax, 1
		mov     byte ptr [edx+ecx], al
		sub     ebp, eax
		mov     dword ptr ds:46E3D8h, 0
		mov     eax, 434D63h
		jmp     eax
FULL:
		mov     eax, ebx
		and     eax, 0FFh
		shl     eax, 1
		mov     byte ptr [edx+ecx], al
		sub     ebx, eax
		sub     ebp, eax
		mov     dword ptr [esp+14h], esi
		mov     eax, 434D63h
		jmp     eax
	}
}

//.text:00436311 66 8B 84 71 8C 00 00 00       mov     ax, [ecx+esi*2+8Ch]
//.text:00436319 66 3B C2                      cmp     ax, dx

__declspec(naked) int AssignAllEngineers()
{
	// [esp+60] -> n available engineer
	// [esp+64] -> n available build space
	// [ecx+esi*2+8Ch] -> assigned engineers
	// edx -> 0
	__asm
	{
		mov     ax, [ecx+esi*2+0x8C]
		cmp     ax, dx
		jz		assign_all
		ret
assign_all:
		mov		[esp+32],1
		mov		ebp,1
		mov		ax, [esp+60]
		cmp		ax, [esp+64]
		ja		workspace_full
		mov		[ecx+esi*2+8Ch],ax
		sub		[esp+64],ax
		mov		[esp+60],dx
		cmp		eax,eax
		ret
workspace_full:
		mov		ax, [esp+64]
		mov		[ecx+esi*2+8Ch],ax
		mov		[esp+64],dx
		sub		[esp+60],ax
		cmp		eax,eax
		ret
	}
}

__declspec(naked) void AssignAllEngineers2()
{
	// [esp+60] -> n available engineer
	// [esp+64] -> n available build space
	// [ecx+esi*2+8Ch] -> assigned engineers
	// edx -> 0
	__asm
	{
		//mov     ax, word ptr [ecx+esi*2+8Ch]
		cmp     ax, dx
		jz      AssignAll
		dec     eax
		dec     eax
		mov     ebp, 1
		mov     word ptr [ecx+esi*2+8Ch], ax
		mov     ecx, dword ptr [esp+38h]  //available engineers
		mov     eax, dword ptr [esp+3Ch] //WrkSpace
		inc     ecx
		inc     ecx
		inc     eax
		mov     dword ptr [esp+38h], ecx
		mov     dword ptr [esp+3Ch], eax
		call    WaitTilMouseButtonsAreReleased
		mov     ecx, 43633Eh
		jmp     ecx
AssignAll:
//		mov     dword ptr [esp+32h],1
		mov     ebp, 1
		mov     eax, dword ptr [esp+38h]
		mov     ecx, dword ptr [esp+3Ch]
		and     eax, 0FFh
		and     ecx, 0FFh
		shr     eax, 1
		cmp     cx, ax
		jb      FULL
		sub     dword ptr [esp+3Ch], eax
		shl     eax, 1
		sub     dword ptr [esp+38h], eax     //[esp+38h] -> n available engineer
		mov     ecx, dword ptr ds:49AAD0h
	//	and     eax, 0FFh                    //[esp+3Ch] -> n available build space
		mov     word ptr [ecx+esi*2+8Ch], ax //[ecx+esi*2+8Ch] -> assigned engineers
		mov     ecx, 43633Eh
		jmp     ecx
FULL:
		mov     eax, dword ptr [esp+3Ch]
		mov     dword ptr [esp+3Ch], edx
		shl     eax, 1
		mov     ecx, dword ptr ds:49AAD0h
		and     eax, 0FFh
		mov     word ptr [ecx+esi*2+8Ch], ax
		sub     dword ptr [esp+38h], eax
		mov     ecx, 43633Eh
		jmp     ecx
	}
}

void AutoSell()
{
	InsertCall(0x4363D5, handleDecrementProductionNumber);
	Nop(0x4363DA, 41); //Nop out the unused bytes.
	InsertCall(0x44175A, jumpToProductionCompleteHandler);
	Nop(0x44175F, 1); //Nop out the unused bytes.
	InsertCall(0x436151, drawNewManufactureQuantityDialogLabel);
	InsertCall(0x4351A8, drawNewMainManufactureQuantityLabel);
	InsertJmp(0x435278, drawNewManufactureDialogEntry);
	Nop(0x435278+5,70);
}

__declspec(naked) void ScientistLimits()
{
	__asm
	{
		mov     edx, dword ptr [esp+20h]   //arg_0]
		cmp     bp, 1
		jle     skip
		mov     eax, dword ptr ds:49AB30h
		mov     dword ptr [esp+10h], esi  //var_C]
		inc     byte ptr [edx+eax]
		inc     byte ptr [edx+eax]
		dec     ebp                //scientists
		dec     ebp
		dec     ebx   //labspace
skip:
		mov     eax, 434D34h
		jmp     eax           //loc_434D34
	}
}

__declspec(naked) void ScientistLimits2()
{
	__asm
	{
		cmp     ecx, 78h
		jl      skip
		cmp     ecx, 86h
		jg      skip
		mov     ecx, dword ptr ds:49AB30h
		mov     al, byte ptr [edx+ecx]
		test    al, al
		jz      skip
		dec     al
		dec     al
		inc     ebp             
		inc     ebp             // scientist
		mov     byte ptr [edx+ecx], al
		inc     ebx
		mov     dword ptr [esp+14h], esi //var_8]
		skip:
		mov     eax, 434D63h
		jmp     eax
	}
}

__declspec(naked) void ScientistLimits3()
{
	__asm
	{
		and     edi, 0FFFFh
		shr     edi, 1
		sub     eax, edi
		mov     word ptr ds:49B1C2h, di
		mov     edi, 434E41h
		jmp     edi
	}
}

__declspec(naked) void ScientistLimits4()
{
	__asm
	{
		movzx   dx, byte ptr [ecx+edi+0C0h]
		and     edx, 0FFFh
		shr     edx, 1
		cmp     dx, ax
		jl      moretime
		mov     edx, 441CE2h
		jmp     edx
		moretime:
		mov     ecx, 441D34h
		jmp     ecx
	}
}

__declspec(naked) void EngineerLimits()
{
	__asm
	{
		cmp     word ptr [esp+38h], 1 //engineers left
		jle     next
		cmp     edi, edx
		jge     skip
		inc     eax
		inc     eax           //num engineers  
		mov     dword ptr [esp+1Ch], 1  //var_18
		mov     word ptr [ecx+esi*2+8Ch], ax
		mov     ecx, dword ptr [esp+38h] //engineers left
		mov     eax, dword ptr [esp+3Ch] //WrkSpace
		dec     ecx
		dec     ecx
		dec     eax
		mov     dword ptr [esp+38h], ecx
		mov     ecx, dword ptr ds:49AAD0h //BPROD.DAT
		mov     dword ptr [esp+3Ch], eax
	skip:
		mov     edx, 4362FDh
		jmp     edx
	next:
		mov     ecx, 4363B4h
		jmp     ecx
	}
}

__declspec(naked) void EngineerLimits2()
{
	__asm
	{
		cmp     ax, dx
		jz     skip
		dec     eax
		dec     eax
		mov     ebp, 1
		mov     word ptr [ecx+esi*2+8Ch], ax
		mov     ecx, [esp+38h]  //baseIndex
		mov     eax, dword ptr [esp+3Ch] //WrkSpace
		inc     ecx
		inc     ecx
		inc     eax
		mov     dword ptr [esp+38h], ecx
		mov     dword ptr [esp+3Ch], eax
	skip:
		mov     ecx, 43633Eh
		jmp     ecx
	}
}

__declspec(naked) void EngineerLimits2b()
{
	__asm
	{
		and     edx, 0FFFh
		shr     edx, 1
		movsx   ecx, dx
		and     eax, 0FFFFh
		cdq
		idiv    ecx
		mov     esi, 435241h
		jmp     esi
	}
}

__declspec(naked) void EngineerLimits3()
{
	__asm
	{
		mov    bx, word ptr ds:49B21Eh
		and    ebx, 0FFFFh
		shr    ebx, 1
		mov    word ptr ds:49B21Eh, bx
		add    word ptr ds:49B228h, bx
		mov    cx, word ptr ds:49B228h
		sub    eax, ecx
		mov    ebx, 435AF3h
		jmp    ebx
	}
}

__declspec(naked) void EngineerLimits4()
{
	__asm
	{
		movsx   edx, word ptr [ebp-8Ch] 
		shr     edx, 1
		sub     eax, edx
		test    eax, eax
		mov     [edi], eax
		jg      skip
		mov     edx, 4416DCh
		jmp     edx
	skip:
		mov     edx, 4418B1h
		jmp     edx
	}
}

__declspec(naked) void PurchaseLimits()
{
	__asm
	{
		cmp   word ptr 470750h[eax], 3
		jb    skip
		dec   ecx
	skip:
		dec   ecx
		mov   word ptr 470760h[eax], cx
		mov   edx, 4367ADh
		jmp   edx
	}
}

__declspec(naked) void PurchaseLimits2()
{
	__asm
	{
		cmp   word ptr 470750h[eax*4], 3
		jb    skip
		inc   ecx
	skip:
		inc   ecx
		mov   word ptr 470760h[eax*4], cx
		mov   eax, 436717h
		jmp   eax
	}
}

__declspec(naked) void BaseDefensesAmmoCheck()
{
	__asm
	{
		cmp     dword ptr ds:46E3D8h, 2
		jz      skip2
		cmp     ax, 6
		jnz     laser
		mov     edx, dword ptr ds:49B110h
		sub     edx, 0C350h
		mov     dword ptr ds:49B110h, edx
		mov     ebp, 3
		mov     ecx, 4485C9h
		jmp     ecx
	laser:
		cmp     ax, 9
		jnz     plasma
		mov     ebp, 7
		mov     ecx, 4485C9h
		jmp     ecx
	plasma:
		cmp     ax, 0Ah
		jnz     fusion
		mov     ebp, 8
		mov     ecx, 4485C9h
		jmp     ecx
	fusion:
		mov     edx, dword ptr [esp+14h] //baseoffset
		cmp     word ptr [edx+0D0h], 0
		jz      skip
		mov     ecx, 4
		cmp     word ptr [edx+0D0h], 4
		jge     ammogood
		movzx   ecx, word ptr [edx+0D0h]
	ammogood:
		sub     word ptr [edx+0D0h], cx
		mov     ebp, 6
		mov     ecx, 4485C9h
		jmp     ecx
	skip:
		//add     ebx, 8
		mov     edx, dword ptr ds:47AE58h
		push    1
		push    10h
		push    ebx
		push    0DBh
		push    edx
		call    DrawString
		push    1
		push    09Eh //
		push    ebx
		push    0DBh
		mov     edx, dword ptr ds:47ABA0h //CF0h
		push    edx
		call    DrawString
		push    1
		call    __mkgmtime
		add     esp, 2Ch
skip2:
		push    32h
		call    Sleep_time
		add     esp, 4
		mov     ecx, 448707h
		jmp     ecx
	}
}

__declspec(naked) void BaseDefensesUFOResponse()
{
	__asm
	{
		cmp     byte ptr [edi+63h], 80h
		jz      end
		add     ebx, 8
		push    64h
		call    Sleep_time
		add     esp, 4
		push    1
		push    10h
		push    ebx
		push    0DBh
		mov     eax, dword ptr ds:47B224h
		push    eax
		call    DrawString
		push    1
		call    __mkgmtime
		push    64h
		call    Sleep_time
		add     esp, 1Ch 
		mov     eax, 545943h
		mov     ecx, 484Fh
		add     eax, ecx
		mov     edx, dword ptr [esp+2Ch] //2=grav shield present
		mov     ecx, 46h
		cmp     byte ptr [edi], 0Bh
		jnz     skip1
		mov     ecx, 50h
skip1:
		push    064h
		call    GetRandomNum
		cmp     dl, 2
		jnz     noshield
		mov     cx, 28h
noshield:
		cmp     ax, cx
		push    1
		push    0DCh
		push    ebx
		jnb     MISS
		//HIT
		push    2Eh 
		mov     eax, dword ptr ds:47A730h
		push    eax
		call    DrawString
		push    1
		call    __mkgmtime
		add     esp, 1Ch
		push    0A0h
		push    3Fh
		push    0Bh
		call    SoundCall
		push    64h
		call    Sleep_time
		add     esp, 10h
		mov     edx, dword ptr [esp+10h]  //current y-axis count
		mov     eax, dword ptr [esp+20h]  //current row starting byte
		add     edx, eax
		mov     eax, dword ptr [esp+14h]  //base offset
		add     eax, edx
		mov     byte ptr [eax+16h], 0FFh
		mov     byte ptr [eax+3Ah], 0FFh
		jmp     end
	MISS:
		push    0DBh
		mov     eax, dword ptr ds:47A738h
		push    eax
		call    DrawString
		push    1
		call    __mkgmtime
		push    64h
		call    Sleep_time
		add     esp, 20h
	end:
		add     ebx, 8
		cmp     bx, 96h
		jle     skip
		mov     eax, 4486E0h
		jmp     eax
	skip:
		mov     eax, 448707h
		jmp     eax
	}
}

__declspec(naked) void RetaliationMsg()
{
	__asm
	{
		cmp     byte ptr [edi+63h], 80h
		jnz     skip
		push    1
		push    10h
		push    ebx
	    push    2Eh 
		mov     eax, dword ptr ds:47B2D4h //47B21Ch  
		push    eax
		call    DrawString
		push    1
		call    __mkgmtime
		add     esp, 18h
		push    19h
		push    3Fh
		push    0Ah
		call    SoundCall
		add     esp, 0Ch
		jmp     end
   	skip:
		mov   liftypos, 0  //41B66Ch
		mov   liftxpos, 0
		mov   al, byte ptr [edi]
		cmp   al, 0Bh
		jnz   end
		mov   dword ptr [esp+10h], 0
		mov   esi, dword ptr [esp+14h]
		lea   esi, [esi+16h]
		loopstart:
		mov   al, byte ptr [esi]
		test  al, al
		jnz   next
		mov   eax, dword ptr [esp+10h]
		xor   edx, edx
		mov   ebp, 6//ebx, 6
		idiv  ebp //ebx
		mov   ebp, 0Ah//ebx, 0Ah
		imul  ebp //ebx
		add   eax, 4
		mov   liftypos, eax  //xpos of airlock
		mov   ecx, dword ptr [esp+10h]
		xor   edx, edx
		mov   eax, ecx
		mov   ebp, 6//ebx, 6
		idiv  ebp//ebx
		imul  ebp//ebx
		sub   ecx, eax
		mov   eax, ecx
		mov   ebp, 0Ah//ebx, 0Ah
		imul  ebp//ebx
		add   eax, 4
		mov   liftxpos, eax  //ypos of airlock
		jmp   msg
	next:
		mov   eax, dword ptr [esp+10h]
		inc   al
		cmp   al, 24h
		jz    end
		mov   dword ptr [esp+10h], eax
		inc   esi
		jmp   loopstart
	msg:
		add     bl, 8
		push    1
		push    10h
		push    ebx
		push    2Eh 
		mov     eax, dword ptr ds:47B2DCh //47B21Ch  
		push    eax
		call    DrawString
		push    1
		call    __mkgmtime
		add     esp, 18h
		push    19h
		push    3Fh
		push    0Ah
		call    SoundCall
		add     esp, 0Ch 
end:	
		push   32h
		call   Sleep_time
		mov    edx, 4487ADh
		jmp    edx
	}
}

__declspec(naked) void BaseDefensePrep()
{
	__asm  //44EA44h
	{
		mov     edx, dword ptr ds:49AAB0h
		mov     ecx, [esp+30h] //curlocLOCDATmemadd
		xor     eax, eax
		xor     esi, esi
		mov     al, byte ptr [ecx+edx+1]
		mov     dword ptr [esp+10h], 32h //counter
		lea     ecx, [eax+eax*8]
		lea     edx, [eax+ecx*4]
		mov     eax, dword ptr ds:49AAD8h //pBASEDAToffset
		lea     edi, [eax+edx*4]
		mov     dword ptr [esp+1Ch], edi  //base_ref
	locloopstart:
		mov     ecx, dword ptr ds:49AAB0h
		cmp     byte ptr [esi+ecx], 2
		jnz     skip
		mov     edx, ecx
		xor     eax, eax
		mov     al, byte ptr [esi+edx+1]
		lea     ecx, [eax+eax*2]
		lea     edx, [eax+ecx*4]
		mov     eax, dword ptr ds:49AB18h //pCraftDat
		mov     ecx, dword ptr [esp+4Ch] //curlocLOCDATidx
		cmp     [eax+edx*8+1Ah], cx
		lea     eax, [eax+edx*8] 
		jnz     skip
		mov     cx, word ptr [eax+2Ah]
		cmp     cx, 1
		jz      skip
		cmp     cx, 0FFFFh
		jz      skip
		lea     ecx, [eax+2Ch]
		lea     eax, [edi+74h]
		mov     edx, 37h
	invloopstart:
		movzx   di, byte ptr [ecx]
		mov		byte ptr [ecx], 0
		add     word ptr [eax], di
		add     eax, 2
		inc     ecx
		dec     edx
		jnz     invloopstart
	skip:
		mov     eax, dword ptr [esp+10h] //counter
		add     esi, 14h
		dec     eax
		mov     dword ptr [esp+10h], eax //counter
		jnz     locloopstart
		mov     edi, dword ptr [esp+4] //curlocLOCDATidx
		mov     eax, dword ptr ds:49AAB0h
		movzx   eax, byte ptr[edi+eax+1]
		push    2
		call    LoadGeodataPalette
		push    2
		call    LoadGeodataBackPalette
		push    1
		call    LoadGeodataBackPalette
		add     esp, 0Ch
		push    eax
		call    ShowBaseEditor
		add     esp, 4
		push    5
		call    LoadGeodataBackPalette
		add     esp, 4
		mov     esi, 44EADBh //Fh
		jmp     esi
	}
}

__declspec(naked) void BaseDefenseBriefing()
{
	__asm
	{   
		push    0A0h
		push    3Fh
		push    86h
		call    SoundCall
		add     esp, 0Ch
		push    1
		call    LoadGeographBackScreen
		push    1
		call    LoadGeographBackPalette
		push    0F0h
		push    140h
		push    0C8h
		push    0
		push    0
		call    DrawDialogBox
		mov     eax, dword ptr ds:47A768h
		push    0
		push    10h
		push    18h
		push    86h
		push    eax
		call    DrawString
		mov     eax, dword ptr ds:47A76Ch
		add     esp, 30h
		push    0
		push    10h
		push    28h
		push    86h
		push    eax
		call    DrawString
		add     esp, 14h
		movsx   eax, word ptr [ebp+10h]
		mov     ecx, dword ptr ds:49AAB0h
		lea     eax, [eax+eax*4]
		movzx   dx, byte ptr [ecx+eax*4+1]
		lea     ecx, [edx+edx*8]
		lea     eax, [edx+ecx*8]
		shl     eax, 2
		xor     edx,edx
		mov     ecx, dword ptr ds:49AAD8h //++
		add     ecx, eax
		push    0
		push    0FFFFh
		push    28h
		push    86h
		push    ecx        //++
		call    DrawString
		add     esp, 14h
		mov     eax, dword ptr ds:47A770h
		push    1
		push    10h
		push    40h
		push    86h
		push    eax
		mov     word ptr ds:4782B0h, 120h
		call    DrawString
		add     esp, 14h
		mov     ecx, dword ptr ds:47A4D4h
		push    ecx
		push    86h
		push    78h
		push    12h
		push    64h
		push    0A4h
		call    CreateButton
		add     esp, 18h
		push    1
		call    __mkgmtime
		add     esp, 4
		call    WaitTilMouseButtonsAreReleased
		loc_44D01D:
		call    GetClickedButton
		test    ax, ax
		jz      loc_44D01D 
		movzx   eax, word ptr [ebp+10h]
		mov     ecx, dword ptr ds:49AAB0h
		add     esp, 4
		mov     edx, 45792Ah //457935h
		jmp     edx
	}
/*	int basename=(*(int*)0x49AAD8)+(basenum*292);
	SoundCall(0x86,0x3F,0xA0);
	*(short*)0x49B214=3;
	LoadGeographBackScreen(1);
	LoadGeodataBackPalette(1);
	DrawDialogBox(0,0,0xC8,0x140,0xF0);
	DrawString2(*(int*)0x47A768,0x86,0x18,0x10,0); 
	DrawString2(*(int*)0x47A76C,0x86,0x28,0x10,0);
	DrawString((char*)basename,0x86,0x28,0xFFFFFFFF,0);
	*(short *)0x4782B0=0x120;
	DrawString2(*(int*)0x47A770,0x86,0x40,0x10,1); 
	CreateButton(0xA4,0x64,0x12,0x78,0x86,(char*)0x47A4D4);
	__mkgmtime(1);
	WaitTillMouseButtonsAreReleased();
	while(!CheckForClickedButton());
	((void (*)(int, int))(0x44CD40))(basenum, locnum);*/
}

__declspec(naked) void engineernotify()
{
	__asm
	{
		push    5
		call    LoadGeographBackScreen
		push    1
		call    LoadGeographBackPalette
		push    0F0h
		push    120h
		push    0B4h
		push    10h
		push    0Ah
		call    DrawDialogBox
		add     esp, 1Ch
		mov     edi, dword ptr ds:47A44Ch //You will need...
		push    0
		push    0
		push    20h 
		push    0F0h
		push    edi
		mov     word ptr ds:4782AEh, 0A0h 
		//mov     word ptr ds:4782B2h, 5Ah
		mov     word ptr ds:4782B0h, 10Ch
		call    DrawString2
		add     esp, 14h
		mov     eax, dword ptr ds:47AAECh   //Sectoid
		push    0
		push    0
		push    38h 
		push    0F0h
		push    eax
		mov     word ptr ds:4782AEh, 0A3h
		//mov     word ptr ds:4782B2h, 5Ah
		mov     word ptr ds:4782B0h, 10Ch
		call    DrawString2
		add     esp, 14h
		mov     eax, dword ptr ds:47AB20h //Engineer
		push    0
		push    0
		push    48h
		push    0F0h
		push    eax
		mov     word ptr ds:4782AEh, 0A0h
		mov     word ptr ds:4782B0h, 10Ch
		call    DrawString2
		add     esp, 14h
		mov     eax, dword ptr ds:47B1F4h  //In order to ..
		push    0
		push    0
		push    50h
		push    0F0h
		push    eax
		mov     word ptr ds:4782AEh, 0A0h
		mov     word ptr ds:4782B0h, 110h
		call    DrawString2
		add     esp, 14h
		mov     eax, dword ptr ds:47ABA8h //mind shield
		push    0
		push    0
		push    78h
		push    0F0h
		push    eax
		mov     word ptr ds:4782AEh, 0A0h
		mov     word ptr ds:4782B0h, 10Ch
		call    DrawString2
		add     esp, 14h
		mov     ecx, dword ptr ds:47A4D4h
		push    ecx
		push    86h
		push    0A0h
		push    12h
		push    50h
		push    96h
		call    CreateButton
		push    1
		call    __mkgmtime
		add     esp, 1Ch
		call    WaitTilMouseButtonsAreReleased
		pop     ebp
		loc_4451ED:
		call    GetClickedButton
		test    ax, ax
		jz      loc_4451ED
		mov     edi, 2
		add     esp, 14h
		retn
	}
}

__declspec(naked) void HyperwaveMod()
{
	__asm
	{		
		mov   bl, byte ptr [esi+0Ch]
		test  bl, bl
		jnz   skip
		mov   byte ptr [esi+0Ch], 1
	skip:
		mov   bx, word ptr ds:49B85Eh  //Current difficulty
		test  bl, bl
		jz    finalcheck
		mov   dl, byte ptr ds:470140h  //sectoid nav
		test  dl, dl
		jz    skip1
		dec   bl
		test  bl, bl
		jz    unlock
	skip1:
		mov   dl, byte ptr ds:4701C4h  //snakeman
		test  dl, dl
		jz    skip2
		dec   bl
		test  bl,bl
		jz    unlock
	skip2:
		mov   dl, byte ptr ds:4702CCh  //muton
		test  dl, dl
		jz    skip3
		dec   bl
		test  bl,bl
		jz    unlock
	skip3:
		mov   dl, byte ptr ds:470350h  //floater
		test  dl,dl
		jz    finalcheck
		dec   bl
	finalcheck:
		test  bl, bl
		jz    unlock
		jmp   end
	unlock:
		push  29h
		call  UnlockResearch
		add   esp,4
	end:
		mov   bl,3
		push  ebp
		push  6
		call  UnlockUFOpaediaTopic
		add   esp, 8
		mov   ecx, 44475Fh
		jmp   ecx
	}
}

__declspec(naked) void ResearchMod()
{
	__asm  //444B67
	{
		test  byte ptr ds:4704D4h, 1
		jz    unlockprod
		test  byte ptr ds:4704F8h, 1
		jz    unlockprod
		test  byte ptr ds:47051Ch, 1
		jz    unlockprod
		cmp   si, 52h
		jnz   LASER
		test  byte ptr ds:46FBEAh, 1  //hv plasma
		jnz   cannonunlock
		test  byte ptr ds:46FC16h, 1  //plasma rifle
		jz    unlockprod
cannonunlock:
		push  21h
		call  UnlockResearch
		add   esp, 4
unlockprod:
		test  byte ptr ds:46FBEAh, 1  //hv plasma
		jz    rifle
		test  byte ptr ds:46FC00h, 1  //hv clip
		jz    rifle
		push  31h
		call  UnlockProduction
		add   esp, 4
		push  32h
		call  UnlockProduction
		add   esp, 4
	rifle:
		test  byte ptr ds:46FC16h, 1  //rifle
		jz    pistol
		test  byte ptr ds:46FC2Ch, 1  //clip
		jz    pistol
		push  33h
		call  UnlockProduction
		add   esp, 4
		push  34h
		call  UnlockProduction
		add   esp, 4
	pistol:
		test  byte ptr ds:46FC42h, 1  //pistol
		jz    LASER
		test  byte ptr ds:46FC58h, 1  //clip
		jz    LASER
		push  35h
		call  UnlockProduction
		add   esp, 4
		push  36h
		call  UnlockProduction
		add   esp, 4
LASER:
		test  byte ptr ds:470420h, 1 //in Prod.dat 1=can't produce
		jnz   BLASTER
		test  byte ptr ds:46FCDCh, 1  //E-115
		jz    BLASTER
		test  byte ptr ds:46FD08h, 1 //ufo power
		jz    BLASTER
		test  byte ptr ds:46FE3Ch, 1 //Hv laser
		jz    BLASTER
		push  04h
		call  UnlockProduction
		push  0Ch
		call  UnlockProduction
		add   esp, 8
BLASTER:
		test  byte ptr ds:470552h, 1 //can already produce bombs
		jz    probe
		test  byte ptr ds:46FC84h, 1
		jz    probe
		cmp   si, 40h
		jz    bombproduce
		cmp   si, 46h
		jz    bombproduce
		cmp   si, 52h
		jnz   probe
bombproduce:
		push  38h
		call  UnlockProduction
		add   esp, 4
probe:
		test   byte ptr ds:47059Ah, 1  //can produce mind probe?
		jz     mindshield
		test   byte ptr ds:46FCF2h, 1
		jz     mindshield
		and    esi, 0FFFFh
		lea    ecx, [esi+esi*4]
		lea    eax, [esi+ecx*2]
		lea    ecx, ds:46FB88h[eax*2]
		cmp    byte ptr [ecx+4], 3  //Is current completed project an alien engineer?
		jnz    mindshield
		push   3Dh
		call   UnlockProduction
		add    esp, 4
mindshield:
		cmp    si, 27h
		jnz    engineercheck
		call   engineernotify
engineercheck:
		cmp    si, 40h  //sectoid engineer
		jnz    NEXT
		test   byte ptr ds:46FEECh, 1
		jz     NEXT
		mov    [esp+10h], 0Dh  //var_4
NEXT:
		mov    esi, 444BDEh
		jmp    esi
	}
}

__declspec(naked) void AltPsionicReasearchTree()
{
	__asm
	{
		cmp    di, 2
		jz     etheral
		test   di, di
		jnz    skip
		cmp    byte ptr [esi+4], bl
		jnb    skip
		jmp    sectoid
	etheral:    
		push   28h
		call   UnlockResearch
		add    esp, 4
	sectoid:
		push   27h
		call   UnlockResearch
		add    esp,4
	skip:
		mov    edx, 4447CFh
		jmp    edx
	}
}

__declspec(naked) void AltLaserResearchHeavy()
{
	__asm
	{
		push   1Eh                //unlock rifle after heavy laser
		call   UnlockResearch     
		push   20                //unlock laser cannon after heavy
		call   UnlockResearch
		add    esp, 8
		push   21h
		call   UnlockProduction
		add    esp, 4
		mov    edx, 444AB4h
		jmp    edx
	}
}

void AltLaserWeaponResearch()
{
	InsertJmp(0x444A38,AltLaserResearchHeavy);
	char rifle[]={0x1D};
	PatchInPlace(0x444A1D,rifle,1);	//pistol after rifle
	Nop(0x444A02,5);				//remove call after finish pistol
	char heavy[]={0x1F};
	PatchInPlace(0x4449EF,heavy,1);	//heavy laser after general laser finished
}


